__doc__ = '''FunDB for local dict structured database for python


   -> Easy to use
   -> Offline
   -> python dictnary format

IMPORT AND INTI:
    >>> from fundb import fdb
    >>> db = fdb('nameOfDb',  'secret',  rotate = 8000, layer = 1)

        > db_Name = 'nameOfDb'   [required]
        > password = 'secret'    [optional]
        > rotate = 8000          [optional]
        > layer = 1              [optional]  # slows the process
                                             According to the layer.
        > rrotate = 6            [optional]

CREATE DB:
   -> It over write data
   -> (type) dict only

    >>> Data = {'name': 'cat', 'age': 2}
    >>> db.write(Data)

        > data = Data
        > verbose = False

READ DATA:
   -> (type) dict only

    >>> db.read()
    {'name': 'cat', 'age': 2}

INSERT PARRENT KEY:
    >>> db.insert('key1', ['value1', 'value2'])
    {'name': 'cat', 'age': 2, 'key1': ['value1', 'value2']}

        > key = 'key1'
        > value = ['value1', 'value2']

REMOVE PARRENT KEY:
    >>> db.remove('key1')
    {'name': 'cat', 'age': 2}

        > key = 'key1'

GET VALUE FROM PARRENT KEY:
    >>> db.getval('age')
    2

        > key = 'age'

SEARCH VALUE:
    >>> tmp = db.search('ca')
    >>> list[tmp]
    [{'name': 'cat'}]

        > string = 'ca'

DB INFO:
    >>> db.db_info()
    {'Name': 'nameOfDb.dbf', 'Created': 'Fri Mar 12 07:52:32 2021', 'Modifyed': 'Fri Mar 12 07:52:32 2021', 'Size': 120, 'Mode': 33152, 'UId': 1000, 'GId': 1000}

FREE READ AND WRITE:
   -> Insert Any format

    >>> Data2 = 'String type'
    >>> db.free_write(Data2)

        > data = Data2

    >>> db.free_read()
    'String type'

'''

from fundb.fundb import Fundb as fdb




from milkycow import *
from codelib.base import base64encode as benc
from codelib.base import base64decode as bdec
from re import search, ASCII
from getpass import getpass as ainput
from os import remove, stat
from time import ctime



RDATA = {
        "FNF" : "\u001b[31m[!] DBNotFound:\u001b[0m Please create data base with write().",
        "DWV" : "[✓] Data written db: {} bytes: {} data: {}.",
        "UDE" : "\u001b[31m[!] InvaliedPassword:\u001b[0m Unable to get data.",
        "TYE" : "\u001b[31m[!] InvaliedData:\u001b[0m Unable to get dictnary data use free_read().",
        "TYE2": "\u001b[31m[!] InvaliedData:\u001b[0m read() accepts only dictnary data use free_write().",
        "KEE" : "\u001b[31m[!] InvaliedKey:\u001b[0m key '{}' is not found in this database.",
        "TEI" : "\u001b[31m[!] InvaliedKey:\u001b[0m insert() only works with dictnary data.",
        "TER" : "\u001b[31m[!] InvaliedKey:\u001b[0m remove() only works with dictnary data.",
        "INE" : "\u001b[31m[!] InvaliedDBName:\u001b[0m Please Use valied database name.",
        "EXP" : [ "", None, ".", ".."]
        }



class Fundb:
    def __init__(self, db_name, password = "pass", rotate = 500, layer=0, rrotate=6):
        if(db_name in RDATA["EXP"]):
            raise Exception(RDATA["INE"])
        self.db_name = db_name
        self.passwd = password
        self.rotate = rotate
        self.layer = layer
        self.rrotate = rrotate
        self.exten = ".dbf"
        if(self.db_name == ""):
            raise Exception(RDATA["FNF"])

    def write(self, data ,verbose = False):
        """
        >>> # The data will be over write
        >>> # pass the data in dict
        >>> mdb.write({"name": "foo"})
        """
        if(type(data) != dict):
            raise Exception(RDATA["TYE2"])
        with open(self.db_name+self.exten, "w") as a:
            enc_data = encrypt(benc(str(data)), self.passwd, self.rotate, self.layer, self.rrotate)
            a.write(enc_data)
            a.close()
            if(verbose):
                return RDATA["DWV"].format(self.db_name, len(enc_data), data)
            else:
                return data

    def read(self):
        """
        >>> # It read only in dict format
        >>> mdb.read()
        """
        try:
            with open(self.db_name+self.exten, "r") as a:
                data = decrypt(a.read(), self.passwd, self.rotate, self.layer, self.rrotate)
                a.close()
                return eval(bdec(data))
        except UnicodeDecodeError:
            raise Exception(RDATA["UDE"])
        except FileNotFoundError:
            raise Exception(RDATA['FNF'])
        except TypeError:
            raise Exception(RDATA["TYE"])
        except:
            raise Exception(RDATA["TYE"])

    def free_write(self, data , verbose = False):
        """
        >>> # The data will be over write
        >>> # write what ever you want. use free_read() to get tha data back
        >>> mdb.free_write("data")
        """
        with open(self.db_name+self.exten, "w") as a:
            enc_data = encrypt(benc(str(data)), self.passwd, self.rotate, self.layer, self.rrotate)
            a.write(enc_data)
            a.close()
            if(verbose):
                return RDATA["DWV"].format(self.db_name, len(enc_data), data)
            else:
                return data

    def free_read(self):
        """
        >>> # It reads any format and return as str
        >>> mdb.free_read()
        """
        try:
            with open(self.db_name+self.exten, "r") as a:
                data = decrypt(a.read(), self.passwd, self.rotate, self.layer, self.rrotate)
                a.close()
                return bdec(data)
        except UnicodeDecodeError:
            raise Exception(RDATA["UDE"])
        except FileNotFoundError:
            raise Exception(RDATA["FNF"])

    def rmdb(self):
        return remove(self.db_name+self.exten)

    def db_info(self):
        """
        >>> # data base Information
        >>> mdb.db_info()
        """
        data = list(stat(self.db_name+self.exten))
        info = {"Name": self.db_name+self.exten}
        info["Created"] = ctime(data[-3])
        info["Modifyed"] = ctime(data[-1])
        info["Size"] = data[-4]
        info["Mode"] = data[0]
        info["UId"] = data[4]
        info["GId"] = data[5]
        return info

    def insert(self, key, value):
        try:
            data = self.read()
            data[key] = value
            self.write(data)
            return self.read()
        except TypeError:
            raise Exception(RDATA["TEI"])


    def remove(self, key):
        try:
            data = self.read()
            del data[key]
            self.write(data)
            return self.read()
        except KeyError:
            raise Exception(RDATA["KEE"].format(key))
        except TypeError:
            raise Exception(RDATA["TER"])



    def close(self):
        self.db_name = ""
        self.passwd = ""




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fundb",
    version="0.0.1",
    author="Madhava",
    author_email="alformint@gmail.com",
    description="Simple database for Python user",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Madhava-mng/FunDB",
    use2to3 = True,
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires = ["milkycow"]
)
